@@ -27,6 +27,12 @@ namespace exactextract {
 
         Coordinate(double p_x, double p_y) : x{p_x}, y{p_y} {}
 
+        double distance(const Coordinate &other) const {
+            double dx = other.x - x;
+            double dy = other.y - y;
+            return std::sqrt(dx*dx + dy*dy);
+        }
+
         bool equals(const Coordinate &other, double tol) const {
             return std::abs(other.x - x) < tol && std::abs(other.y - y) < tol;
         }
